#
#  ZopeEditController.py
#  ZopeEditManager
#
#  Created by Zachery Bir on 2003-07-18.
#  Copyright (c) 2003 Zope Corporation. All rights reserved.
#

# library imports
import os, re
from objc import YES, NO
from PyObjCTools import NibClassBuilder

# import needed classes/functions from Foundation
from Foundation import *

# import Nib loading functionality from AppKit
from AppKit import *

# local product imports
from ZopeDocument import ZopeDocument

# create ObjC classes as defined in MainMenu.nib
NibClassBuilder.extractClasses("MainMenu")
class ZopeEditController(NibClassBuilder.AutoBaseClass):
    """
    The Controller object for managing various processes editing Zope
    objects externally.
    """
    def updateIfModified(self, timer):
        timerUser = timer.userInfo()
        for doc in timerUser.current_edits_data:
            mtime = os.path.getmtime(doc.getContentFile())

            if mtime != doc.last_mtime:
                # File was modified
                doc.putChanges()
                doc.last_mtime = mtime

        return 1
            
    def application_openFile_(self, app, filename):
        """
        Invoked by NSApplication when the app attempts to open a file
        """

        zopeDoc = ZopeDocument(filename)
        self.current_edits_data.append(zopeDoc)
        self.current_edits.reloadData()

        editor = zopeDoc.getEditor()
        content_file = zopeDoc.getContentFile()

        if zopeDoc.useLocks():
            zopeDoc.lock()

        return self.ws.openFile_withApplication_andDeactivate_(content_file, 
                                                               editor, YES)

    def finishEdits_(self):
        """
        We want to get rid of the lock on a Zope object and Delete its
        local copy.
        """
        row = self.current_edits.selectedRow()
        if row != -1:
            if NSRunAlertPanel("Deleting Entry",
                               "Are you sure you're finished with this"
                               " object?",
                               "Yes", "Cancel", None):
                del self.current_edits_data[row]
                self.current_edits.reloadData()

    def numberOfRowsInTableView_(self, tableView):
        return len(self.current_edits_data)

    def tableView_objectValueForTableColumn_row_(self, tableView,
                                                 aColumn, aRow):
        ident = aColumn.identifier()
        return getattr(self.current_edits_data[aRow], 'get' + ident)()
        
    def init(self):
        self.current_edits_data = []
        self.ws = NSWorkspace.sharedWorkspace()
        NSUserDefaults.resetStandardUserDefaults()
        sud = NSUserDefaults.standardUserDefaults()
        sud.registerDefaults_(
            NSDictionary.dictionaryWithContentsOfFile_(
                NSBundle.mainBundle().pathForResource_ofType_(
                    NSBundle.mainBundle().bundleIdentifier(), "plist")))
        interval = (sud.floatForKey_('save_interval')) or 20.0
        timer = (
    NSTimer.scheduledTimerWithTimeInterval_target_selector_userInfo_repeats_( 
                interval, self, 'updateIfModified', self, YES))
        NSRunLoop.currentRunLoop().addTimer_forMode_(timer, 
                                                     'NSDefaultRunLoopMode')

        return self

